'use strict';

exports.__esModule = true;
exports.default = getRealPath;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _log = require('./log');

var _mapToRelative = require('./mapToRelative');

var _mapToRelative2 = _interopRequireDefault(_mapToRelative);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function findPathInRoots(sourcePath, _ref) {
  var extensions = _ref.extensions,
      root = _ref.root;

  // Search the source path inside every custom root directory
  var resolvedSourceFile = void 0;

  root.some(function (basedir) {
    resolvedSourceFile = (0, _utils.nodeResolvePath)(`./${sourcePath}`, basedir, extensions);
    return resolvedSourceFile !== null;
  });

  return resolvedSourceFile;
}

function getRealPathFromRootConfig(sourcePath, currentFile, opts) {
  var absFileInRoot = findPathInRoots(sourcePath, opts);

  if (!absFileInRoot) {
    return null;
  }

  var realSourceFileExtension = _path2.default.extname(absFileInRoot);
  var sourceFileExtension = _path2.default.extname(sourcePath);

  // Map the source and keep its extension if the import/require had one
  var ext = realSourceFileExtension === sourceFileExtension ? realSourceFileExtension : '';
  return (0, _utils.toLocalPath)((0, _utils.toPosixPath)((0, _utils.replaceExtension)((0, _mapToRelative2.default)(opts.cwd, currentFile, absFileInRoot), ext)));
}

function checkIfPackageExists(modulePath, currentFile, extensions) {
  var resolvedPath = (0, _utils.nodeResolvePath)(modulePath, currentFile, extensions);
  if (resolvedPath === null) {
    (0, _log.warn)(`Could not resolve "${modulePath}" in file ${currentFile}.`);
  }
}

function getRealPathFromAliasConfig(sourcePath, currentFile, opts) {
  var aliasedSourceFile = void 0;

  opts.alias.find(function (_ref2) {
    var regExp = _ref2[0],
        substitute = _ref2[1];

    var execResult = regExp.exec(sourcePath);

    if (execResult === null) {
      return false;
    }

    aliasedSourceFile = substitute(execResult);
    return true;
  });

  if (!aliasedSourceFile) {
    return null;
  }

  if (aliasedSourceFile[0] === '.') {
    return (0, _utils.toLocalPath)((0, _utils.toPosixPath)((0, _mapToRelative2.default)(opts.cwd, currentFile, aliasedSourceFile)));
  }

  if (process.env.NODE_ENV !== 'production') {
    checkIfPackageExists(aliasedSourceFile, currentFile, opts.extensions);
  }

  return aliasedSourceFile;
}

var resolvers = [getRealPathFromRootConfig, getRealPathFromAliasConfig];

function getRealPath(sourcePath, _ref3) {
  var file = _ref3.file,
      opts = _ref3.opts;

  if (sourcePath[0] === '.') {
    return sourcePath;
  }

  // File param is a relative path from the environment current working directory
  // (not from cwd param)
  var currentFile = _path2.default.resolve(file.opts.filename);
  var resolvedPath = null;

  resolvers.some(function (resolver) {
    resolvedPath = resolver(sourcePath, currentFile, opts);
    return resolvedPath !== null;
  });

  return resolvedPath;
}